#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#include "jtypes.h"
#include "softbank.h"
#include "vidmode.h"
#include "palfile.h"
#include "genpal.h"


// genPalette()     by Kenneth Hsu
// everything else  by John Beppu


/*

%% .................................................................. */
u32 parseCommands (
    int argc,
    char *argv[],
    char **palfile,
    char **directory,
    int **actionLUT
    )
{
    u32 u0,u1;
    u32 len;
    u32 pSet = 0;
    u32 dSet = 0;

    (*actionLUT) = (int*) calloc(argc,sizeof(int*));
    if ((*actionLUT) == NULL) return 5;

    for (u0 = 1; u0 < (u32) argc; u0++) {
        u1 = strcmp(argv[u0],"-p");
        if ((u1 == 0) || (strcmp(argv[u0],"-d") == 0)) {
            if (u1 == 0) {
                u0++;
                if (pSet) return 1;
                if (u0 < (u32) argc) {
                    len = strlen(argv[u0]);
                    (*palfile) = (char*) malloc(len * sizeof(u8));
                    if ((*palfile) == NULL) return 5;
                    strcpy((*palfile),argv[u0]);
                    pSet = 1;
                }
                else return 1;
            }
            else {
                u0++;
                if (dSet) return 2;
                if (u0 < (u32) argc) {
                    len = strlen(argv[u0]);
                    (*directory) = (char*) malloc(len * sizeof(u8));
                    if ((*directory) == NULL) return 5;
                    strcpy((*directory),argv[u0]);
                    dSet = 1;
                }
                else return 2;
            }
        }
        else {
            (*actionLUT)[u0] = 1;
        }
    }
    if (!pSet) (*palfile)   = NULL;
    if (!dSet) (*directory) = NULL;
    return 0;
}


/*
    [function]  Makes sure the directory *dest exists (creating the
                directory if necessary, and makes sure the directory
                can be written to.
    [returns]   0:success, 1:couldn't mkdir(), 2:not writable
%% .................................................................. */
u32 validateDir(char *dest)
{
    char    *currentDir = NULL;
    char    *tempName;
    u32     u0,u1;

    tempName = (char*) malloc(L_tmpnam * sizeof(char));
    if (tempName == NULL) return 1;
    tempName = tmpnam(tempName);
    currentDir = getcwd(currentDir,1024);
    if (currentDir == NULL) {
        free(tempName);
        return 1;
    }
    
    u0 = chdir(dest);
    if (u0) {                           // try to mkdir()
        u1 = mkdir(dest,0777);
        if (u1) {
            free(tempName);
            free(currentDir);
            return 1;
        }
    }
    else {                              // make sure it's writable
        FILE *testFile;
        testFile = fopen(tempName,"wb+");
        if (testFile == NULL) {
            free(tempName);
            free(currentDir);
            return 2;
        }
        remove(tempName);
        fclose(testFile);
        chdir(currentDir);
    }
    free(tempName);
    free(currentDir);
    return 0;
}


/*

%% .................................................................. */
void ripSprites (
    char *dest,
    char *filename,
    u8 *pal,
    u32 palLEN
    )
{
    u32		u0,u1,u2;
    char        *HSI_name = NULL;
    char	*HSI_id_number;
    static u32  id = 0;
    
    // setup destination

    u0 = strlen(dest);
    u1 = strlen(filename);
    u2 = u0 + u1 + 11;
    HSI_name = (char*) malloc(u2 * sizeof(u8));
    if (HSI_name == NULL) throw("BAD_ALLOC");
    strcpy(HSI_name,dest);
    HSI_name[u0] = '/';
    HSI_id_number = &HSI_name[u0+1];

    FILE        *HSI_file;
    static u8   HSI_buffer[0x40000];    // a measly 256k is safe

    try {
        SoftBank sb(filename);
        u1 = sb.spriteQty();
        u2 = 0;
        for (u0 = 0; u0 < u1; u0++) {
            u2 = (u2 > sb.sizeOfHSI(u0)) ? u2 : sb.sizeOfHSI(u0);
        }
        sb.definePalette(pal,palLEN);
        for (u0 = 0; u0 < u1; u0++) {
            printf("%04x : [%2d/%2d] %-20s (",id,u0+1,u1,filename);
            printf("w = %d :",sb.spriteWd(u0));
            printf("h = %d)\n",sb.spriteHi(u0));
            sprintf(HSI_id_number,"%04x.raw",id++);
            sb.makeHSI(HSI_buffer,u0);
            HSI_file = fopen(HSI_name,"wb");
            fwrite(HSI_buffer,sizeof(u8),sb.sizeOfHSI(u0),HSI_file);
            fclose(HSI_file);
        }
    }
    catch (char *n) {
        printf("Something bad (aka %s) happened.\n",n);
    }
    if (HSI_name != NULL)   free(HSI_name);
}


/*

%% .................................................................. */
int main (
    int argc,
    char *argv[]
    )
{
    u32     u0;
    char    *p_name = NULL;	// HSIPalFile name
    char    *d_name = NULL;	// destination directory
    int     *actionLUT = NULL;

    // check command line ::::::::::::::::::::::::::::::::::::::::::::::

    if (argc < 2) {
	printf("usage:  %s -p <HSIPalFile> -d <dest.dir> [.spr]\n",
	    argv[0]);
	return 1;
    }
    u0 = parseCommands(argc,argv,&p_name,&d_name,&actionLUT);
    if ((u0 == 1) || (u0 == 2)) {
	printf("usage:  %s -p <HSIPalFile> -d <dest.dir> [.spr]\n",
	    argv[0]);
	return 1;
    }
    if (u0 == 5) {
	printf("memory allocation failed\n");
	if (p_name != NULL) free(p_name);
	if (d_name != NULL) free(d_name);
	if (actionLUT != NULL) free(actionLUT);
	return 2;
    }

    // check destination :::::::::::::::::::::::::::::::::::::::::::::::

    char    currentDir[4] = {"."};

    if (d_name == NULL) d_name = currentDir;
    u0 = strlen(d_name);
    if (d_name[u0-1] == '/') d_name[u0-1] = 0;
    u0 = validateDir(d_name);
    if (u0 == 1) {
        printf("Could not make directory %s\n",d_name);
        return 3;
    }
    if (u0 == 2) {
        printf("%s not writable or not a directory or both\n",d_name);
        return 3;
    }
    
    // palette creation ::::::::::::::::::::::::::::::::::::::::::::::::

    static u8   _pal[768];
    static u32  _palLEN = 0;

    if (p_name != NULL) {
	HSIPalFile colors;
	u0 = colors.load(p_name);
        if (u0) {
            printf("palette file error\n");
            return 4;
        }
        _palLEN = colors.paletteLen();
        u0 = colors.copyPalette24bit(_pal,0,_palLEN);
    }
    else {
        _palLEN = 32;
        genPalette((i8*) _pal,_palLEN);
    }

    #if 1

    // finally, rip sprites ::::::::::::::::::::::::::::::::::::::::::::

    for (u0 = 1; u0 < (u32) argc; u0++) {
	if (actionLUT[u0]) {
            ripSprites(d_name,argv[u0],_pal,_palLEN);
	}
    }

    #endif

    if (p_name != NULL) free(p_name);
    if (d_name != NULL) free(d_name);
    if (actionLUT != NULL) free(actionLUT);
    return 0;
}


